@@ -16,10 +16,8 @@ GVContactsTable::GVContactsTable (QWidget *parent, Qt::WindowFlags flags)
 {
     ui->setupUi (this);
 
-#ifdef Q_WS_MAEMO_5
-    this->setAttribute (Qt::WA_Maemo5StackedWindow);
-    this->setAttribute (Qt::WA_Maemo5AutoOrientation);
-#endif
+    OsDependent &osd = Singletons::getRef().getOSD ();
+    osd.setDefaultWindowAttributes (this);
 
     mnuContext.addAction (ui->actionCall);
     mnuContext.addAction (ui->actionSend_Text);
