@@ -15,10 +15,8 @@ GVHistory::GVHistory (QWidget *parent, Qt::WindowFlags flags)
 {
     ui->setupUi (this);
 
-#ifdef Q_WS_MAEMO_5
-    this->setAttribute (Qt::WA_Maemo5StackedWindow);
-    this->setAttribute (Qt::WA_Maemo5AutoOrientation);
-#endif
+    OsDependent &osd = Singletons::getRef().getOSD ();
+    osd.setDefaultWindowAttributes (this);
 
     // Connect the log to this classes log.
     QObject::connect (
